@@ -23,6 +23,8 @@
 /* Aug 14, 2002 bj: moved these to utils.c */
 /* Jan 13, 2003 ym: added timeout functionality */
 
+/* -------------------------------------------------------------------------- */
+
 typedef void    sigfunc(int);   /* for signal handlers */
 
 sigfunc* sig_catch(int sig, void (*f)(int))
@@ -39,8 +41,9 @@ static void catch_alrm(int x) {
   /* dummy */
 }
 
-ssize_t timeout_read(ssize_t (*reader)(int d, void *buf, size_t nbytes),
-                     int fd, void *buf, size_t nbytes) {
+ssize_t
+fd_timeout_read (int fd, void *buf, size_t nbytes)
+{
   ssize_t nred;
   sigfunc* sig;
 
@@ -50,7 +53,7 @@ ssize_t timeout_read(ssize_t (*reader)(int d, void *buf, size_t nbytes),
   }
 
   do {
-    nred = reader(fd, buf, nbytes);
+    nred = read (fd, buf, nbytes);
   } while(nred < 0 && errno == EAGAIN);
 
   if(nred < 0 && errno == EINTR)
@@ -66,15 +69,48 @@ ssize_t timeout_read(ssize_t (*reader)(int d, void *buf, size_t nbytes),
   return nred;
 }
 
+int
+ssl_timeout_read (SSL *ssl, void *buf, int nbytes)
+{
+  int nred;
+  sigfunc* sig;
+
+  sig = sig_catch(SIGALRM, catch_alrm);
+  if (libspamc_timeout > 0) {
+    alarm(libspamc_timeout);
+  }
+
+  do {
+#ifdef SPAMC_SSL
+    nred = SSL_read (ssl, buf, nbytes);
+#else
+    nred = 0;			// never used
+#endif
+  } while(nred < 0 && errno == EAGAIN);
+
+  if(nred < 0 && errno == EINTR)
+    errno = ETIMEDOUT;
+
+  if (libspamc_timeout > 0) {
+    alarm(0);
+  }
+
+  /* restore old signal handler */
+  sig_catch(SIGALRM, sig);
+
+  return nred;
+}
+
+/* -------------------------------------------------------------------------- */
+
 int
 full_read (int fd, unsigned char *buf, int min, int len)
 {
   int total;
   int thistime;
 
-
   for (total = 0; total < min; ) {
-    thistime = timeout_read (read, fd, buf+total, len-total);
+    thistime = fd_timeout_read (fd, buf+total, len-total);
 
     if (thistime < 0) {
       return -1;
